package com.citygis.gis.tool.domain.component.define;

import com.citygis.gis.tool.api.enums.attribute.AttributeTypeEnum;
import com.citygis.gis.tool.api.enums.attribute.ValueMetaTypeEnum;
import lombok.Getter;
import lombok.Setter;

import java.util.List;

@Getter
@Setter
public class Property extends SimpleProperty {

    /**
     * 默认值
     */
    private Object defaultValue;
    /**
     * 是否可见，0：不可见；1：可见。可用于在模型级别上显示，供最终用户编辑
     */
    private int isVisible = 1;
    /**
     * 备注信息
     */
    private String remark;
    /**
     * 是否只读，1：只读；0：可编辑
     */
    private int readOnly = 0;
    /**
     * 属性值集，提供下拉选择
     */
    private Object[] valueSet;
    /**
     * 是否必须，1：需要；0：可选
     */
    private int isRequired = 1;
    /**
     * 属性值所依赖的数据源
     */
    private String dataSourceCode;
    /**
     * 是否需要前驱依赖
     */
    private int isNeedPrecursor = 0;
    /**
     * 属性类别。0：一般属性；1：运行时属性
     */
    private int type = AttributeTypeEnum.NORMAL.ordinal();
    /**
     * 排序号
     */
    private int sortId;
    /**
     * 依赖的前置编码（这里在模型依赖上使用）
     */
    private List<String> precursorSysCode;

    public Property() {
    }

    /**
     * @param name     名称
     * @param alias    别名
     * @param dataType 数据类型
     * @param value    值
     */
    public Property(
            String name,
            String alias,
            ValueMetaTypeEnum dataType,
            Object value) {
        this.setName(name);
        this.setAlias(alias);
        this.setValue(value);
        this.setDataType(dataType.ordinal());
        this.setDataTypeDesc(dataType.getDesc());
        this.setDataTypeCode(dataType.name());
        this.type = AttributeTypeEnum.NORMAL.ordinal();
    }

    public Property(
            String name,
            String alias,
            ValueMetaTypeEnum dataType,
            Object value,
            String dataSourceCode) {
        this.setName(name);
        this.setAlias(alias);
        this.setValue(value);
        this.setDataType(dataType.ordinal());
        this.setDataTypeDesc(dataType.getDesc());
        this.setDataTypeCode(dataType.name());
        this.type = AttributeTypeEnum.NORMAL.ordinal();
        this.setDataSourceCode(dataSourceCode);
    }

    public Property(
            String name,
            String alias,
            ValueMetaTypeEnum dataType,
            Object value,
            Object defaultValue,
            String remark) {
        this(name, alias, dataType, value);

        this.defaultValue = defaultValue;
        this.valueSet = valueSet;
        this.isVisible = isVisible;
        this.remark = remark;
        this.readOnly = readOnly;
        this.type = AttributeTypeEnum.NORMAL.ordinal();
    }

    public Property(
            String name,
            String alias,
            ValueMetaTypeEnum dataType,
            Object value,
            Object defaultValue,
            String remark,
            Object[] valueSet,
            int isVisible,
            int readOnly,
            int required) {
        this(name, alias, dataType, value);

        this.defaultValue = defaultValue;
        this.valueSet = valueSet;
        this.isVisible = isVisible;
        this.remark = remark;
        this.readOnly = readOnly;
        this.isRequired = required;
        this.type = AttributeTypeEnum.NORMAL.ordinal();
    }

    public Property(
            String name,
            String alias,
            ValueMetaTypeEnum dataType,
            Object value,
            Object defaultValue,
            String remark,
            Object[] valueSet,
            int isVisible,
            int readOnly,
            int required,
            String dataSourceCode,
            int isNeedPrecursor) {
        this(name, alias, dataType, value);

        this.defaultValue = defaultValue;
        this.valueSet = valueSet;
        this.isVisible = isVisible;
        this.remark = remark;
        this.readOnly = readOnly;
        this.isRequired = required;
        this.dataSourceCode = dataSourceCode;
        this.isNeedPrecursor = isNeedPrecursor;
        this.type = AttributeTypeEnum.NORMAL.ordinal();
    }

    public Property(
            String name,
            String alias,
            ValueMetaTypeEnum dataType,
            Object value,
            Object defaultValue,
            String remark,
            Object[] valueSet,
            int isVisible,
            int readOnly,
            int required,
            String dataSourceCode,
            int isNeedPrecursor,
            List<String> precursorSysCode) {
        this(name, alias, dataType, value);

        this.defaultValue = defaultValue;
        this.valueSet = valueSet;
        this.isVisible = isVisible;
        this.remark = remark;
        this.readOnly = readOnly;
        this.isRequired = required;
        this.dataSourceCode = dataSourceCode;
        this.isNeedPrecursor = isNeedPrecursor;
        this.type = AttributeTypeEnum.NORMAL.ordinal();
        this.precursorSysCode = precursorSysCode;
    }

    public Property(
            String name,
            String alias,
            ValueMetaTypeEnum dataType,
            Object value,
            Object defaultValue,
            String remark,
            Object[] valueSet,
            int isVisible,
            int readOnly,
            int required,
            String dataSourceCode,
            int isNeedPrecursor,
            AttributeTypeEnum attributeType) {
        this(name, alias, dataType, value, defaultValue, remark, valueSet, isVisible, readOnly, required, dataSourceCode, isNeedPrecursor);
        this.type = attributeType.ordinal();
    }

    /**
     * 覆写ToString方法
     *
     * @return
     */
    @Override
    public String toString() {
        if (null == getValue()) {
            return "";
        }
        return this.getValue().toString();
    }

    /**
     * 设置属性唯一编码
     *
     * @param name
     * @return
     */
    public Property initName(String name) {
        this.setName(name);
        return this;
    }

    /// <summary>
    /// 设置属性别名
    /// </summary>
    /// <param name="alias"></param>
    /// <returns></returns>
    public Property initAlias(String alias) {
        this.setAlias(alias);
        return this;
    }

    /// <summary>
    /// 设置属性的数据类型
    /// </summary>
    /// <param name="dataType"></param>
    /// <returns></returns>
    public Property initDataType(ValueMetaTypeEnum dataType) {
        this.setDataType(dataType.ordinal());
        this.setDataTypeCode(dataType.name());
        this.setDataTypeDesc(dataType.getDesc());
        return this;
    }

    /**
     * 设置属性值
     *
     * @param value
     * @return
     */
    public Property initValue(Object value) {
        super.setValue(value);
        return this;
    }

    /**
     * 设置默认值
     *
     * @param defaultValue
     * @return
     */
    public Property initDefaultValue(Object defaultValue) {
        this.defaultValue = defaultValue;
        return this;
    }

    /**
     * 设置预置的值
     *
     * @param valueSet
     * @return
     */
    public Property initValueset(Object[] valueSet) {
        this.valueSet = valueSet;
        return this;
    }

    /**
     * 设置是否可见
     *
     * @param isVisible 1：可见；0：不可见
     * @return
     */
    public Property initVisible(int isVisible) {
        this.isVisible = isVisible;
        return this;
    }

    /**
     * 设置备注值
     *
     * @param remark
     * @return
     */
    public Property initRemark(String remark) {
        this.remark = remark;
        return this;
    }

    /**
     * 是否只读
     *
     * @param readOnly 1：只读；0：可编辑
     * @return
     */
    public Property initReadOnly(int readOnly) {
        this.readOnly = readOnly;
        return this;
    }

    /**
     * 是否必须。默认为1
     *
     * @param required 1：需要；0：可选
     * @return
     */
    public Property initRequired(int required) {
        this.isRequired = required;
        return this;
    }

    /**
     * 属性依赖的数据源编码
     *
     * @param dataSourceCode
     * @return
     */
    public Property initDataSourceCode(String dataSourceCode) {
        this.dataSourceCode = dataSourceCode;
        return this;
    }

    /**
     * 是否需要前驱依赖
     *
     * @param isNeedPrecursor 1：是；0：否
     * @return
     */
    public Property initNeedPrecursor(int isNeedPrecursor) {
        this.isNeedPrecursor = isNeedPrecursor;
        return this;
    }

    /**
     * 属性类别
     *
     * @param attributeType 0：一般属性；1：运行时属性
     * @return
     */
    public Property initType(AttributeTypeEnum attributeType) {
        this.type = attributeType.ordinal();
        return this;
    }

    /**
     * 设置排序号
     *
     * @param sortId
     * @return
     */
    public Property initSortId(int sortId) {
        this.sortId = sortId;
        return this;
    }

    /**
     * 设置前置依赖的模型编码集合
     *
     * @param precursorSysCode
     * @return
     */
    public Property initPrecursorSyscode(List<String> precursorSysCode) {
        this.precursorSysCode = precursorSysCode;
        return this;
    }
}
